//==============================================================================
// THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES INC. ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT, ARE
// DISCLAIMED. IN NO EVENT SHALL ANALOG DEVICES INC. BE LIABLE FOR ANY DIRECT,
// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// YOU ASSUME ANY AND ALL RISK FROM THE USE OF THIS CODE OR SUPPORT FILE.
//
// IT IS THE RESPONSIBILITY OF THE PERSON INTEGRATING THIS CODE INTO AN
// APPLICATION TO ENSURE THAT THE RESULTING APPLICATION PERFORMS AS REQUIRED
// AND IS SAFE.
//
//     Module       : main.c
//     Description  :
//     Date         : 16. March 2012
//     Version      : v1.00
//     Changelog    : v1.00 Initial
//
//     Measure temperature with PT100 RTD on-baord
//     Output via UART 9600 Baud, 8 Bit, No Parity, 1 Stop, No Handshake
//     Connection from AGND to -REFIN is required!
//
//==============================================================================
#include "ADuCM360.h"
#include <stdio.h>

void __putchar(char ch);
void SendString(char *cString);
void delay(int mSec);

//==============================================================================
unsigned char ucString[100];

int iADC1RESULT = 0;
float RValue = 0.0;
float TValue = 0.0;
float VValue = 0.0;

//==============================================================================
// main to flash LED approx. 250ms by SW delay
//==============================================================================
int main (void)
{
	// Initialise
	T3CON = 0;  					// Turn off watchdog - is on by default after POR !!!

	// Configure Clocks correctly - key for low power !
	CLKSYSDIV = 0;					// Core clock set to 16MHz - normally it is 8MHz (1) for low power
	CLKCON0 = (7 << 5)|			 	// Clockout = LFOSC for low current operation
              (0 << 3)|				// Clockin  = HFOSC (default)
			  (0 << 0);				// Clockdiv = CD = 0 => 16MHz

	CLKCON1 = (7 << 12)|			// PWM CLK 	= 125kHz => UCLK/128 = UCLK/2^7
			  (0 <<  9)|			// UART CLK = 16MHz  => UCLK/1
			  (7 <<  6)|			// I2C  CLK = 125kHz
			  (7 <<  3)|			// SPI1 CLK	= 125kHz
			  (7 <<  0);			// SPI2 CLK	= 125kHz

	CLKDIS	= (0 << 9)|				// ADC CLK disable = 1 / 0 = enable
			  (1 << 8)|				// DMA CLK
			  (1 << 7)|				// DAC CLK
			  (1 << 6)|				// Timer1 CLK
			  (1 << 5)|				// Timer0 CLK
			  (1 << 4)|				// PWM  CLK
			  (0 << 3)|				// UART CLK
			  (1 << 2)|				// I2C  CLK
			  (1 << 1)|				// SPI1 CLK
			  (1 << 0);				// SPI2 CLK

	// Configure GPIO P0 for UART
	GP0CON = (3 << (1*2))|		    // P0.1 as RX
			 (3 << (2*2));  	    // P0.2 as TX

  // Configure UART for 9600 Baud
	COMCON = 0;						// UART enable / 1 = disable

	COMLCR = (0 << 6)|				// Normal Mode
			 (0 << 5)|				// No Parity Force
			 (0 << 4)|				// Odd Parity
			 (0 << 3)|				// No Parity
			 (0 << 2)|				// 1 Stop Bits
			 (3 << 0);				// 8 Bit

	COMDIV = 17;

	COMFBR = (  1 << 15)|			// Enable Fractional Baud Rate Generator
			 (  3 << 11)|			// DIVM
			 (131 <<  0);			// DIVN


	// Configure GPIO P1 for LED
	GP1CON = 0x00000000;            // all P1 pin as GPIOs
	GP1OEN = (1 << 3);				// enable P1.3 as output - LED

	GP0PUL = (1 << 3);				// enable P1.3 pull-up

	GP0OUT = (1 << 3);				// P1.3 OFF - LED

	// Init ADC1
	ADC1MDE = 0x03;					// ADC1 into Idle mode
	ADC1MSKI = 0x0;					// No IRQ

	ADC1CON = (1 << 19)|			// enable ADC1
              (0 << 18)|			// bipolar coding
              (0 << 17)|			// - Buffer enabled
              (0 << 16)|			// + Buffer enabled
              (0 << 15)|            // + Buffer used
              (0 << 14)|			// - Buffer used
              (1 << 12)|			// external reference
              (0 << 10)|            // Diagnostic Currents Off
              (0 <<  5)|			// +ve input AIN0
              (1 <<  0);			// -ve input AIN1


    ADCCFG  = 0x01;					// External reference buffers on

	ADC1FLT = (1 << 15)|			// Chop ON
              (1 << 14)|			// RAVG2
              (0 << 12)|			// SINC4 OFF
              (15<<  8)|			// AF
              (0 <<  7)|			// NOTCH2 OFF
              (63 << 0);			// SF

	ADC1MDE = 0x33;					// PGA = 4 / Idle Mode
	// Enable IEXC0, Internal IREF,  AIN6 = IEXC0(used for RTD)
	IEXCCON = (0 << 7)|				// Power up IEXC
              (1 << 6)|				// Select internal current source
              (0 << 3)|				// IEXC1 off
              (6 << 0);				// IEXC0 on

	IEXCDAT = 0x0E;					// Init IEXC0 200A

	ADC1MDE = 0x21;					// Enable continuous conversions on ADC1 with PGA = 4

	while(1)
    {
		if(ADC1STA & 0x01)          // ADC1 Ready
		{
            iADC1RESULT = ADC1DAT;  // Read RTD raw value

            // Calculate Values for R, T & V
			// Ratiometric Calculation of the RTD resistance
			//RValue = ((float)iADC1RESULT * 5600.0)/268435455.0;
			RValue = ((float)iADC1RESULT * 5656.0)/268435455.0; // Wrong Rref

			// In the range between 0C and 100C the linear equation
			// R = Ro * (1 + a * T)
			// with Ro = 100 Ohm / a = 0.00385/K
			// can be used
			TValue = ((RValue/100.0) - 1.0)/0.00385;

			// Just for reference the voltage across the RTD
			VValue = (float)(iADC1RESULT * 5600.0*0.0002)/268435455.0;


			sprintf((char *)ucString,"\r %08X R:%6.3f V:%8.6f Temperature:%6.3fC ",iADC1RESULT, RValue, VValue, TValue);
			SendString((char *)ucString);
			GP1TGL = (1 << 3);  	// Toggle P1.3 - LED
		}
    }
}

//==============================================================================
// Send a Character to UART
//==============================================================================
void __putchar(char ch)
{
    while(!(COMLSR & (1 << 5))){}	// Wait till TX Buffer is empty
									// Possibly before TX shift is empty - bit 6 of COMLSR
	COMTX = ch;
}

//==============================================================================
// Send a String via SendChar
//==============================================================================
void SendString(char *cString)
{
	while(*cString != 0)
  	{
        __putchar((char)*cString++);
	}
}

//==============================================================================
// General delay code. Delays in mSec.
//==============================================================================
void delay(int mSec)
{
  	int d1, d2;
  	d1 = mSec;
  	d2 = mSec;
  	for ( d1 = 0; d1 < mSec; d1++ )
  	{
    	for ( d2 = 0; d2 < 2340; d2++ )
    	{
    	}
  	}
}

//==============================================================================
